from flask_restful import Resource
from flask import jsonify
from lyrebird import application


class Event(Resource):

    def get(self, channel=None, page=0):
        db = application.server['db']
        channel_rules = []
        if channel:
            channel_rules = channel.split('+')
        events = db.get_event(channel_rules, offset=page*20)
        result = []
        for event in events:
            event_str = event.json()
            result.append(event_str)
        return jsonify(result)


class Channel(Resource):

    def get(self):
        db = application.server['db']
        channel_list = db.get_channel_list()
        return jsonify([item[0] for item in channel_list])



class DataManagerError(Exception):
    pass


class DataRootDirNotExistsError(DataManagerError):
    pass


class DataGroupNotExistsError(DataManagerError):
    pass


class DataGroupInfoNotFoundError(DataManagerError):
    pass


class DataGroupIsNotDirError(DataManagerError):
    pass


class DataNotExistsError(DataManagerError):
    pass


class DataIsNotDirError(DataManagerError):
    pass


class DataInfoNotFoundError(DataManagerError):
    pass


class ActivateFailed(DataManagerError):
    pass




from flask import Response


class RequestPathNotFound:

    def handle(self, handler_context):
        if not handler_context.response:
            handler_context.response = Response('Request path not found\n', 404)



from .manager import main, run
from .mock import context
from .mock.plugin_manager import PluginView, caller_info
from .mock.handlers.handler_context import HandlerContext
from .mock.reporter.report_handler import report
from .mock import plugin_manager
from blinker import Signal
import os
from .event import CustomEventReceiver
from lyrebird import application
from lyrebird.log import get_logger


APPLICATION_CONF_DIR = os.path.join(os.path.expanduser('~'), '.lyrebird')


event  = CustomEventReceiver()


def start_background_task(target, *args, **kwargs):
    """Start a background task in a new thread

    :param target: task function
    :param args: args will be pass to the task function
    :param kwargs: kwargs will be pass to the task function
    :return: null
    """
    context.application.socket_io.start_background_task(target, *args, **kwargs)


def emit(event, *args, **kwargs):
    """
    Send socketio event

    """
    context.application.socket_io.emit(event, *args, **kwargs)


def get_plugin_storage():
    """
    Get plugins storage dir path

    :return: ~/.lyrebird/plugins/<plugin_name>
    """
    info = caller_info(index=2)
    storage_name = info.top_module_name
    plugin_storage_dir = os.path.abspath(os.path.join(APPLICATION_CONF_DIR, 'plugins/%s' % storage_name))
    if not os.path.exists(plugin_storage_dir):
        os.makedirs(plugin_storage_dir)
    return plugin_storage_dir


def subscribe(channel, func, *args, **kwargs):
    """
    订阅信号

    :param signal: 信号，包含lyrebird, overbridge, android, tracking, perf, hunter
    :param func: 响应事件的回调函数
    :param sender: 信号发送者标识
    """
    # context.application.event_bus.subscribe(channel, func)
    application.server['event'].subscribe(channel, func, *args, **kwargs)


def publish(channel, event, *args, **kwargs):
    """
    发送信号

    :param signal: 信号，包含lyrebird, overbridge, android, tracking, perf, hunter
    :param sender: 信号发送者标识
    :param kwargs: 事件数据
    :return:
    """
    # context.application.event_bus.publish(channel, event)
    application.server['event'].publish(channel, event, *args, **kwargs)


def add_background_task(name, func):
    """
    添加后台任务

    """
    application.server['task'].add_task(name, func)


def get_plugin_conf():
    info = caller_info(index=2)
    plugin_name = info.top_module_name
    return plugin_manager.get_conf(plugin_name)


"""
获取 Event server state
"""
class StateProxy:

    def get(self, index):
        return application.server['event'].state.get(index)

    def __getitem__(self, index):
        return application.server['event'].state[index]

state = StateProxy()



from lyrebird import HandlerContext
from flask import Response


class MyDataHandler:

    def handle(self, handler_context: HandlerContext):
        if 'www.baidu.com' in handler_context.request.path:
            print('Got "baidu" in request url')
            if handler_context.response:
                origin_data = handler_context.response.get_data().decode()
                new_data = origin_data.replace('百度', 'google')
                new_data = new_data.replace('<img hidefocus="true" src="//www.baidu.com/img/bd_logo1.png" width="270" height="129">',
                                            '<div><h1>Google</h1></div>')
                handler_context.response.set_data(new_data.encode())
        elif 'meituan' in handler_context.request.path:
            print('Got "meituan" in request url')
            handler_context.response = Response('Hello', 404)
        else:
            print(f'Got url {handler_context.request.url[:100]}')



import json
import datetime
import traceback
from queue import Queue
from lyrebird import application
from lyrebird import log
from lyrebird.base_server import ThreadServer
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, String, Integer, Text, DateTime, create_engine

"""
Database server

Worked as a backgrund thread
storage all event message into database
"""

logger = log.get_logger()

Base = declarative_base()


class LyrebirdDatabaseServer(ThreadServer):
    def __init__(self):
        super().__init__()

        DB_FILE_NAME = 'lyrebird.db'
        ROOT_DIR = application.root_dir()
        SQLALCHEMY_DATABASE_URI = ROOT_DIR/DB_FILE_NAME
        sqlite_path = 'sqlite:///'+str(SQLALCHEMY_DATABASE_URI)

        engine = create_engine(str(sqlite_path))
        # Create all tables
        Base.metadata.create_all(engine)
        # Create session factory
        session_factory = sessionmaker(bind=engine)
        Session = scoped_session(session_factory)
        self._scoped_session = Session

        # init queue
        self.storage_queue = Queue()

        # subscribe all channel
        application.server['event'].subscribe('any', self.event_receiver)

    def event_receiver(self, event, channel=None):
        content = json.dumps(event)
        flow = Event(channel=channel, content=content)
        self.storage_queue.put(flow)

    def run(self):
        session = self._scoped_session()
        while self.running:
            try:
                event = self.storage_queue.get()
                session.add(Event(channel=event.channel, content=event.content))
                session.commit()
            except Exception:
                logger.error(f'Save event failed. {traceback.format_exc()}')

    def stop(self):
        super().stop()

    @property
    def session(self):
        return self._scoped_session()

    def get_event(self, channel_rules, offset=0, limit=20):
        session = self._scoped_session()
        if len(channel_rules) == 0:
            result = session.query(Event) \
                .order_by(Event.id.desc()) \
                .offset(offset) \
                .limit(limit) \
                .all()
        else:
            result = session.query(Event) \
                .order_by(Event.id.desc()) \
                .filter(Event.channel.in_(channel_rules)) \
                .offset(offset) \
                .limit(limit) \
                .all()
        self._scoped_session.remove()
        return result

    def get_channel_list(self):
        session = self._scoped_session()
        result = session.query(Event.channel) \
            .group_by(Event.channel) \
            .all()
        self._scoped_session.remove()
        return result


class JSONFormat:

    def json(self):
        prop_collection = {}
        props = dir(self)
        for prop in props:
            if prop.startswith('_'):
                continue
            prop_obj = getattr(self, prop)
            if isinstance(prop_obj, (str, int, bool)):
                prop_collection[prop] = prop_obj
            elif isinstance(prop_obj, datetime.datetime):
                prop_collection[prop] = str(prop_obj)
        return prop_collection


class Event(Base, JSONFormat):
    __tablename__ = 'flow'

    id = Column(Integer, primary_key=True, autoincrement=True)
    channel = Column(String(16), index=True)
    content = Column(Text)
    ts = Column(DateTime(timezone=True), default=datetime.datetime.now)




from flask_restful import Resource
from lyrebird.mock import context
from flask import request, jsonify, abort


class MockGroup(Resource):
    """
    mock数据组
    """
    def get(self, group_id=None):
        if not group_id:
            groups = context.application.data_manager.groups
            return jsonify([groups[group_id].json() for group_id in groups])

        # 返回数据组中数据列表
        _group = context.application.data_manager.groups.get(group_id)
        if not _group:
            return context.make_fail_response('Not found group')

        return jsonify(_group.json(detail=True))

    def post(self):
        name = request.json.get('name')
        parent = request.json.get('parent')
        group = context.application.data_manager.create_group()
        group.name = name
        group.parent_id = parent
        group.save()
        return context.make_ok_response(group_id=group.id)

    def put(self):
        name = request.json.get('name')
        group_id = request.json.get('id')
        parent = request.json.get('parent')
        group = context.application.data_manager.groups.get(group_id)
        if not group:
            return context.make_fail_response('Group not found')
        group.name = name
        group.parent_id = parent
        group.save()
        return context.make_ok_response(group_id=group.id)

    def delete(self, group_id):
        context.application.data_manager.delete_group(group_id)
        return context.make_ok_response()


class MockData(Resource):
    """
    mock数据
    """
    def get(self, group_id=None, data_id=None):
        if not group_id or not data_id:
            return abort(400, '请设置参数 group_id & data_id')
        _group = context.application.data_manager.groups.get(group_id)
        if not _group:
            return context.make_fail_response('Group not found')
        _data = _group.all_data.get(data_id)
        if not _data:
            return context.make_fail_response('Data not found')
        return jsonify(_data.json(detail=True))

    def put(self, group_id=None, data_id=None):
        if not group_id or not data_id:
            return abort(400, '请设置参数 group_id & data_id')
        _group = context.application.data_manager.groups.get(group_id)
        if not _group:
            return context.make_fail_response('Group not found')
        _data = _group.all_data.get(data_id)
        if not _data:
            return context.make_fail_response('Data not found')

        _data.name = request.json.get('name')
        _data.rule = request.json.get('rule')
        req = request.json.get('request')
        if req:
            _data.request.content = req['content']
            _data.request.filetype = req['filetype']
        req_data = request.json.get('request_data')
        if req_data:
            _data.request_data.content = req_data['content']
            _data.request_data.filetype = req_data['filetype']
        resp = request.json.get('response')
        if resp:
            _data.response.content = resp['content']
            _data.response.filetype = resp['filetype']
        resp_data = request.json.get('response_data')
        if resp_data:
            _data.response_data.content = resp_data['content']
            _data.response_data.filetype = resp_data['filetype']
        _data.save()
        return context.make_ok_response()

    def post(self, group_id=None, data_id=None):
        name = request.json.get('name')
        rule = request.json.get('rule')
        req = request.json.get('request')
        req_data = request.json.get('request_data')
        resp = request.json.get('response')
        resp_data = request.json.get('response_data')

        _group = context.application.data_manager.groups.get(group_id)
        if not _group:
            return context.make_fail_response(f'Group not found by id={group_id}')

        _data = _group.create_data()
        _data.name = name
        _data.rule = rule
        _data.request.content = req
        _data.request_data.content = req_data
        _data.response.content = resp
        _data.response_data.content = resp_data
        _data.save()
        return context.make_ok_response()

    def delete(self, group_id=None, data_id=None):
        _group = context.application.data_manager.groups.get(group_id)
        if not _group:
            return context.make_fail_response('Group not found')
        ids = request.json.get('ids')
        for data_id in ids:
            _group.delete_data(data_id)
        return context.make_ok_response()


class ActivatedMockGroup(Resource):

    def get(self):
        activated_group_id = context.application.data_manager.activated_group_id
        if not activated_group_id:
            return context.make_fail_response('No group was activated')
        group = context.application.data_manager.groups.get(activated_group_id)
        return context.make_ok_response(name=group.name, id=group.id)

    def put(self, group_id=None, action=None):
        if action == 'activate':
            context.application.data_manager.activate(group_id)
        else:
            context.application.data_manager.deactivate()
        return context.make_ok_response()


class MockGroupByName(Resource):

    def put(self):
        group_name = request.json.get('group_name')
        action = request.json.get('action', 'activate')
        for group_id in context.application.data_manager.groups:
            if group_name == context.application.data_manager.groups[group_id].name:
                if action == 'activate':
                    context.application.data_manager.activate(group_id)
                else:
                    context.application.data_manager.deactivate()
                return context.make_ok_response()
        return context.make_fail_response(f'Group not found. name={group_name}')



from lyrebird.base_server import StaticServer
from . import plugin_loader
from lyrebird import application
from flask import Blueprint, send_file


class PluginManager(StaticServer):

    def __init__(self):
        self.plugin_path_list = []
        self.plugins = {}

    def reload(self):
        self.plugins = {}
        self.plugins.update(plugin_loader.load_all_from_ep())
        for plugin_path in self.plugin_path_list:
            plugin = plugin_loader.load_from_path(plugin_path)
            self.plugins[plugin.project_name] = plugin
        self.setup_plugins()

    def setup_plugins(self):
        # TODO register plugins pb
        for p_name, plugin in self.plugins.items():
            view_static_folder_name = plugin.manifest.view[0]
            view_entry_file = plugin.manifest.view[1]
            plugin_static_folder = f"{plugin.location}/{view_static_folder_name}"

            # Create new blueprint for each plugin
            _bp = Blueprint(
                f'plugins_{p_name}',
                f'plugins_{p_name}',
                url_prefix=f'/plugins/{p_name}',
                static_folder=plugin_static_folder)

            # Add plugin main view to flask blueprint
            _view = plugin.manifest.view
            def view_func():
                return send_file(f"{plugin.location}/{view_static_folder_name}/{view_entry_file}")
            _bp.add_url_rule('/', view_func=view_func)

            # Add API to blureprint
            for api in plugin.manifest.api:
                rule = api[0]
                view_func = api[1]
                if len(api) > 2:
                    methods = api[2]
                else:
                    methods = ['GET']
                _bp.add_url_rule(rule, view_func=view_func, methods=methods)

            # Register blueprint
            mock_service = application.server['mock']
            mock_service.app.register_blueprint(_bp)

            # Subscribe event linstener
            event_service = application.server['event']
            for event_option in plugin.manifest.event:
                channel = event_option[0]
                callback_func = event_option[1]
            event_service.subscribe(channel, callback_func)



"""
checker example

steps:
- ignore unexepcted object
- prepare useful info
- check data

Info used in channel flow
├── name
└─┬ flow
  ├── size
  ├─┬ request
  │ └── url
  └─┬ response
    └─┬ headers
      └── Content-Type

"""


from lyrebird import CustomEventReceiver
from decimal import Decimal


# init event receiver instance
event = CustomEventReceiver()

# THRESHOLD_IMG_SIZE: image size limitation
THRESHOLD_IMG_SIZE = 500

@event('flow')
def img_size(msg):

    # 1.ignore unexepcted object
    if ignore_check(msg):
        return

    # 2.prepare useful info
    img_size = int(msg['flow']['size'])
    img_size = Decimal(img_size / 1024).quantize(Decimal('0.0'))

    # 3.check data
    if img_size > THRESHOLD_IMG_SIZE:
        img_url = msg['flow']['request']['url']
        img_url = img_url[img_url.rfind('//') + 2:]

        description = f'Image size {img_size}KB is beyond expectations: {img_url}\n'
        description += f'Expecte: {THRESHOLD_IMG_SIZE}KB\n'
        description += f'Actual: {img_size}KB\n'
        event.issue(f'Image size {img_size}KB is beyond expectations: {img_url}',
                    {
                        "summary": "[checker][img_size]",
                        "description": description
                    })

def ignore_check(msg):
    if msg['name'] != 'server.response':
        return True
    if 'response' not in msg['flow']:
        return True
    if 'image' not in msg['flow']['response']['headers']['Content-Type']:
        return True
    return False



from flask import jsonify

"""
Lyrebird context

"""

def make_ok_response(**kwargs):
    ok_resp = {
            "code": 1000,
            "message": "success"
        }
    ok_resp.update(kwargs)
    return jsonify(ok_resp)


def make_fail_response(msg):
    return jsonify(
        {
            "code": 3000,
            "message": msg
        }
    )


_cm = None
_src = None


server = {}
plugins = {}


notice = None
checkers = {}

def start_server():
    for name in server:
        server[name].start()


def stop_server():
    for name in server:
        server[name].stop()


class ConfigProxy:

    def get(self, k, default=None):
        return _cm.config.get(k, default)
    
    def __setitem__(self, k, v):
        _cm.config[k] = v

    def __getitem__(self, k):
        return _cm.config[k]

    def raw(self):
        return _cm.config


config = ConfigProxy()


def root_dir():
    if _cm:
        return _cm.ROOT



import imp
import shutil
import importlib
from pathlib import Path
from lyrebird import log
from lyrebird import application
from lyrebird.mock import context
from lyrebird.base_server import ThreadServer
from lyrebird.event import CustomEventReceiver
from lyrebird.mock.reporter.report_handler import report


logger = log.get_logger()

class LyrebirdCheckerServer(ThreadServer):
    def __init__(self):
        super().__init__()

        ROOT = application._cm.root
        self.SCRIPTS_DIR_TEMPLATE = ROOT/'checkers'
        self.EXAMPLE_DIR = Path(__file__).parent/'examples'/'checkers'

        self.scripts_dir = None
        self.checkers = application.checkers
        self.load_checkers()

        # subscribe all channel for ELK
        # TODO: should not subscribe in init
        application.server['event'].subscribe('any', self.report_to_ELK)

    def load_checkers(self):
        # set checkers status for update
        for checker_name in self.checkers:
            self.checkers[checker_name].update = False

        # get checker workspace
        if application.config.get('checker.workspace'):
            self.scripts_dir = Path(application.config.get('checker.workspace'))
            if not self.scripts_dir.exists():
                logger.error('Checker scripts dir not found')
                return
        else:
            self.scripts_dir = self.SCRIPTS_DIR_TEMPLATE
            self.scripts_dir.mkdir(parents=True, exist_ok=True)

        # get checker files
        scripts_list = [script for script in self.scripts_dir.iterdir() if script.name.endswith('.py')]

        # load examples if scripts_dir has no python script
        if not scripts_list:
            for example in self.EXAMPLE_DIR.iterdir():
                if example.name.endswith('.py'):
                    dst = self.SCRIPTS_DIR_TEMPLATE / example.name
                    shutil.copy(example, dst)

        for checker_file in self.scripts_dir.iterdir():
            # Add new checker script
            if checker_file.name.endswith('.py'):
                self.init_checker(checker_file.name, str(checker_file.absolute()))

        del_name_list = []
        for checker_name in self.checkers:
            if not self.checkers[checker_name].update:
                del_name_list.append(checker_name)

        for name in del_name_list:
            del self.checkers[name]

        logger.debug('------ checkers: ------')
        for checker_name in self.checkers:
            # update checker status from config
            switch_conf = application.config.get('checker.switch', {})
            need_activate = switch_conf.get(checker_name, False)

            checker = self.checkers[checker_name]
            if need_activate and not checker.activated:
                checker.activate()
            elif not need_activate and checker.activated:
                checker.deactivate()
            logger.debug(self.checkers[checker_name].json())

    def init_checker(self, name, path):
        if name not in self.checkers:
            checker = Checker(name, path)
            self.checkers[name] = checker
        self.checkers[name].update = True

    def report_to_ELK(self, msg):
        if isinstance(msg, dict) and msg.get('channel') == 'notice':
            msg_sender = msg.get('sender', {})
            report({
                "action": "alert",
                "checker.info": {
                    'module': msg_sender.get('file'),
                    'method': msg_sender.get('function'),
                    'message': msg.get('message')
                }
            })

    def load_scripts(self, scripts):
        for path in scripts:
            try:
                class_module = imp.load_source('checker', path)
                for event in class_module.event.listeners:
                    application.server['event'].subscribe(event['channel'], event['func'])
            except ValueError:
                logger.error(f'{path} failed to load. Only python file is allowed.')

    def run(self):
        super().run()

    def stop(self):
        super().stop()

class Checker:

    def __init__(self, name, path):
        self.name = name
        self.activated = False
        self.path = path
        self.select = False
        self._module = None
        self._event_receiver = None
        self._update = False

    @property
    def update(self):
        return self._update

    @update.setter
    def update(self, val):
        self._update = val

    def activate(self):
        self._module = script_module = self.load_class(self.path)
        event_proxy = getattr(script_module, 'event')
        if isinstance(event_proxy, CustomEventReceiver):
            self._event_receiver = event_proxy
            event_proxy.register(context.application.event_bus)
        self.activated = True

    def deactivate(self):
        if self._event_receiver:
            self._event_receiver.unregister(context.application.event_bus)
            self._event_receiver = None
            self._module = None
        self.activated = False

    def json(self):
        return {k: self.__dict__[k] for k in self.__dict__ if not k.startswith('_')}

    def load_class(self, py_path):
        spec = importlib.util.spec_from_file_location('spec', location=py_path)
        target_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(target_module)
        return target_module



import socket
from pathlib import Path
from colorama import Fore, Style
from .proxy_run import run
from mitmproxy.tools.dump import DumpMaster
from mitmproxy.tools.cmdline import mitmdump
from lyrebird.base_server import ThreadServer
from lyrebird import application
from lyrebird.log import get_logger


"""
HTTP proxy server

Default port 4272
"""

CURRENT_PATH = Path(__file__).parent
FLOW_PATH = CURRENT_PATH/'proxy_flow.py'


logger = get_logger()


class LyrebirdProxyServer(ThreadServer):

    def __init__(self):
        super().__init__()

        conf = application.config
        self.proxy_port = str(conf.get('proxy.port', '4272')) if conf else '4272'
        '''
        --ignore_hosts:
        The ignore_hosts option allows you to specify a regex which is matched against a host:port
        string (e.g. “example.com:443”) of a connection. Matching hosts are excluded from interception,
        and passed on unmodified.

        # Ignore everything but sankuai.com, meituan.com and dianping.com:
        --ignore-hosts '^(?!.*sankuai.*)(?!.*meituan.*)(?!.*dianping.*)'

        According to mitmproxy docs: https://docs.mitmproxy.org/stable/howto-ignoredomains/
        '''
        self.ignore_hosts = None
        if conf.get('proxy.filters'):
            self.ignore_hosts = '^%s' % ''.join(['(?!.*%s.*)' % i for i in conf.get('proxy.filters')])

        self._master = None

    def run(self):
        server_ip = application.config.get('ip')
        # info_msg(f'start on {server_ip}:{self.proxy_port}', f'{Fore.CYAN} ***请在被测设备上设置代理服务器地址***')
        logger.warning(f'start on {server_ip}:{self.proxy_port}   {Fore.CYAN} ***请在被测设备上设置代理服务器地址***')
        mitm_arguments = [
            '-s', str(FLOW_PATH),
            '-p', self.proxy_port,
            '--ssl-insecure',
            '--no-http2',
            '-q'
        ]
        if self.ignore_hosts:
            mitm_arguments += ['--ignore-hosts', self.ignore_hosts]
        run(DumpMaster, mitmdump, mitm_arguments)


def info_msg(*msg):
    print(f'{Fore.YELLOW}[proxy_server]', *msg, Style.RESET_ALL)



import runpy
import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

VERSION = runpy.run_path(
        os.path.join(here, "lyrebird", "version.py")
    )["VERSION"]

with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='lyrebird',
    version=VERSION,
    packages=['lyrebird'],
    url='https://github.com/meituan/lyrebird',
    author='HBQA',
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    zip_safe=False,
    classifiers=(
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
    ),
    entry_points={
        'console_scripts': [
            'lyrebird = lyrebird.manager:main'
        ]
    },
    install_requires=[
        "aniso8601==6.0.0",
        "asn1crypto==0.24.0",
        "astroid==2.2.5",
        "atomicwrites==1.3.0",
        "attrs==19.1.0",
        "beautifulsoup4==4.7.1",
        "blinker==1.4",
        "brotlipy==0.7.0",
        "certifi==2019.3.9",
        "cffi==1.12.2",
        "chardet==3.0.4",
        "Click==6.7",
        "colorama==0.4.1",
        "cryptography==2.3.1",
        "Flask==1.0.2",
        "Flask-RESTful==0.3.7",
        "Flask-SocketIO==3.3.2",
        "h11==0.7.0",
        "h2==3.1.0",
        "hpack==3.0.0",
        "hyperframe==5.2.0",
        "idna==2.8",
        "isort==4.3.16",
        "itsdangerous==1.1.0",
        "Jinja2==2.10",
        "kaitaistruct==0.8",
        "lazy-object-proxy==1.3.1",
        "ldap3==2.5.2",
        "MarkupSafe==1.1.1",
        "mccabe==0.6.1",
        "mitmproxy==4.0.4",
        "more-itertools==7.0.0",
        "packaging==19.0",
        "passlib==1.7.1",
        "pluggy==0.9.0",
        "portpicker==1.3.1",
        "py==1.8.0",
        "pyasn1==0.4.5",
        "pycparser==2.19",
        "pylint==2.3.1",
        "pyOpenSSL==18.0.0",
        "pyparsing==2.2.2",
        "pyperclip==1.6.5",
        "pytest==3.9.3",
        "python-engineio==3.5.0",
        "python-socketio==3.1.2",
        "pytz==2018.9",
        "requests==2.21.0",
        "ruamel.yaml==0.15.89",
        "six==1.12.0",
        "sortedcontainers==2.0.5",
        "soupsieve==1.9",
        "SQLAlchemy==1.3.1",
        "tornado==5.1.1",
        "typed-ast==1.3.1",
        "urllib3==1.24.1",
        "urwid==2.0.1",
        "Werkzeug==0.15.1",
        "wrapt==1.11.1",
        "wsproto==0.11.0"
    ]
    )



from flask_restful import Resource
from lyrebird.mock import context
from flask import request, jsonify, abort
from lyrebird import application


class Notice(Resource):

    def get(self):
        return context.make_ok_response(
            noticeList=application.notice.notice_list,
            notRemindList=application.notice.not_remind_list
        )

    def put(self):
        notice_status = request.json.get('status')
        unique_key = request.json.get('key')
        application.notice.change_notice_status(unique_key, notice_status)
        return context.make_ok_response()

    def post(self):
        msg = request.json.get('eventInfo')

        for action in msg.get('actions'):
            if action.get('type') == 'carrier':

                carrier_message = action.get('box', {})
                issue_channel = carrier_message.get('channel')
                issue_message = carrier_message.get('message')
                issue_state = carrier_message.get('state')

                issue_message['sender'] = msg.get('sender')

                application.server['event'].publish(issue_channel, issue_message, state=issue_state)
        return context.make_ok_response()

    def delete(self):
        unique_key = request.json.get('key')
        application.notice.delete_notice(unique_key)
        return context.make_ok_response()
